/*
 * Copyright (c) 1996, 2000 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*---------------------------------------------------------------------------*
 *
 *	setnetparms - curgets.c
 *	-----------------------
 *
 *	last edit-date: [Fri Jul 27 14:57:29 2001]
 *
 *---------------------------------------------------------------------------*/

#include "defs.h"

/*---------------------------------------------------------------------------*
 *	curgets - return string from a curses box
 *
 *	   str: address of a character array, passed in like &string
 *		this will contain the string which the user inputs
 *	   len: the maximum amount of characters to read in (defines box size)
 *	  y, x: coordinates on the screen of the box's upper left-hand corner
 *	 title: same as str, but it is the title for the box
 *	  hide: hide output, for passwd applications
 *	defstr: if not NULL, string to display as default
 *	  help: if not NULL, string to display as a help message
 *	  type: type of input validation required
 *---------------------------------------------------------------------------*/
void
curgets(char *str, int len, int y, int x, char *title, int hide, char *defstr, char *help, int type)
{
	WINDOW *strwin;
	int c;
	char input[MAXLINELEN];
	int pos;
	int curx;
	int bschar = erasechar();
	int valid = 0;
	
	strwin = newwin(3, len+2, y, x);
	wborder(strwin, 0, 0, 0, 0, 0, 0, 0, 0);
	wstandout(strwin);
	mvwaddstr(strwin, 0, (len / 2) - (strlen(title) / 2), title);
	wstandend(strwin);

	if(help != NULL)
	{
		mvaddstr(LINES-1, 1, help);
		refresh();
	}
	
restart:
	curx = 1;

	if(defstr != NULL)
	{
		strncpy(input, defstr, len);
		input[len] = '\0';
		mvwaddstr(strwin, 1, 1, input);
		curx = strlen(input) + 1;
	}
	else
	{
		for (pos = 1; pos < len; pos++)
			mvwaddch(strwin, 1, pos, ' ');
	}

	for (;;)
	{
		if(curx > len)
			wmove(strwin, 1, curx-1);
		else
			wmove(strwin, 1, curx);
			
		wrefresh(strwin);

		noecho();
		crmode();

		c = wgetch(strwin);

		if(c == bschar)
		{
			if (curx == 1)
				continue;
			mvwaddch(strwin, 1, --curx, ' ');
			input[curx-1] = '\0';
			continue;
		}

		switch(c)
		{
			case KEY_DL:	/* delete line */
				goto restart;
				break;

			case CR:
			case LF:	/* CR or LF */
				input[curx-1] = '\0';
				wclear(strwin);
				wrefresh(strwin);
				delwin(strwin);
				strcpy(str, input);

				if(help != NULL)
				{
					move(23, 1);
					clrtoeol();
					refresh();
				}
				
				return;

			default:
				if (curx > len)
					break;

				if (c < SPACE)
					break;	/* no control chars */

				valid = 0;
				
				switch(type)
				{
					case TYPE_IP:
						if(isdigit(c) || (c == '.'))
							valid = 1;
						break;

					case TYPE_HOST:
						if(isalnum(c) || (c == '-'))
							valid = 1;
						break;

					case TYPE_DOMAIN:
						if(isalnum(c) || (c == '-') || (c == '.'))
							valid = 1;
						break;

					case TYPE_IFF:
						if(isprint(c))
							valid = 1;
						break;
				}
				
				if(!valid)
				{
					beep();
					break;
				}
					
				if (!hide)
					mvwaddch(strwin, 1, curx++, c);
				else
					mvwaddch(strwin, 1, curx++, '*');

				input[curx-2] = c;
				break;
		}
	}
}

/* EOF */
