/*
 * Copyright (c) 1996, 2000 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*---------------------------------------------------------------------------*
 *
 *	setnetparm - menubar.c
 *	----------------------
 *
 * 	based on: Curses menubar
 *	          by Jim King (jek5036@ultb.isc.rit.edu)
 *
 *	last edit-date: [Wed Aug  2 13:50:45 2000]
 *
 *---------------------------------------------------------------------------*/

#include "defs.h"

#define	MAXNAMELEN	70

struct mbar {
	char	menu_choice[MAXNAMELEN];
	int	menu_number;
	struct	mbar	*next;
} *m;

static WINDOW *MENU;

/*---------------------------------------------------------------------------*
 * mkmenubar - converts information in menu to a linked-list
 *---------------------------------------------------------------------------*/
static void
mkmenubar(int *num, char *menu[])
{
	int i = 0;			/* counter for num */
	struct mbar *tmp;		/* tmp pointer to list */

	m = tmp = NULL;			/* init the head and the tmp ptr */

	while(menu[i] != NULL)
	{
		if(!tmp)	/* Empty list -- allocate a new head */
		{
			m = tmp = (struct mbar *)malloc(sizeof(struct mbar));
		}
		else	/* List is not empty -- add to the end of it */
		{
			tmp->next = (struct mbar *)malloc(sizeof(struct mbar));
			tmp = tmp->next;
		}

		if(!tmp)
		{
			move(23, 0);
			refresh();
			endwin();
			perror("mkmenubar: malloc()");
			exit(0);
		}

		tmp->next = NULL;
		strcpy(tmp->menu_choice, menu[i]);
		tmp->menu_number = i+1;
		i++;
	}
	*num = i;	/* 'return' the maxnum of choices */
}

/*---------------------------------------------------------------------------*
 * sizemenubar - determine optimal size for menu bar.
 *---------------------------------------------------------------------------*/
static void
sizemenubar(int *len, int *wid, char *title)
{
	int sz = 0;
	int i = 0;		/* tmp counter */
	struct mbar *tmp;	/* tmp placeholder */

	*len = 0;
	*wid = 0;

	tmp = m;

	for (tmp = m; tmp != NULL; tmp = tmp->next)
	{
		++i;
		sz = strlen(tmp->menu_choice);

		if (sz > *wid)		/* as wide as longest line */
			*wid = sz;
	}

	if (title != NULL)
	{
		if (strlen(title) > *wid)
			*wid = strlen(title);
	}
	
	*wid += 8;		/* +8 for extras like #] and . */
	*len = i+2;		/* +2 for line above and below menu */
}

/*---------------------------------------------------------------------------*
 * dispmenu - sets up the menu on MENU window
 *---------------------------------------------------------------------------*/
static void
dispmenu(int boxflag, char *title, int width, int length)
{
	struct mbar *tmp;

	if (boxflag)
		wborder(MENU, 0, 0, 0, 0, 0, 0, 0, 0);

	if (title != NULL)
	{
		wstandout(MENU);
		mvwaddstr(MENU, 0, (width / 2) - (strlen(title) / 2), title);
		wstandend(MENU);
	}

	for (tmp = m; tmp != NULL; tmp = tmp->next)
	{
		wmove(MENU, tmp->menu_number, 1);
		wprintw(MENU, "%2d - %s", tmp->menu_number, tmp->menu_choice);
	}
	wrefresh(MENU);
}

/*---------------------------------------------------------------------------*
 * delight - un-hilight old selection at num
 *---------------------------------------------------------------------------*/
static void
delight(int num)
{
	struct mbar	*tmp;

	for (tmp = m; tmp != NULL; tmp = tmp->next)
	{
		if (num == tmp->menu_number)
		{
			wmove(MENU, tmp->menu_number, 1);
			wprintw(MENU, "%2d - %s", tmp->menu_number, tmp->menu_choice);
		}
	}
	wrefresh(MENU);
}

/*---------------------------------------------------------------------------*
 * hilight - hilight selection at num
 *---------------------------------------------------------------------------*/
static void
hilight(int num)
{
	struct mbar *tmp;

	for (tmp = m; tmp != NULL; tmp = tmp->next)
	{
		if (num == tmp->menu_number)
		{
			wstandout(MENU);	/* highlight */
			wmove(MENU, tmp->menu_number, 1);
			wprintw(MENU, "%2d - %s", tmp->menu_number, tmp->menu_choice);
			wstandend(MENU);
		}
	}
	wrefresh(MENU);
}

/*---------------------------------------------------------------------------*
 *	menubar - display menu and return selection
 *       y: starting line of menu
 *       x: starting column of menu
 *    menu:  stores the items for be chosen
 * boxflag: if !0, draw box around border of menu
 *   title: address to char array for title of menu
 *    item: inital hilighted bar position
 *   first: if not zero, timeout for anything to enter in seconds
 *	    on timeout, return (-1).
 *---------------------------------------------------------------------------*/
int
menubar(int y, int x, char *menu[], int boxflag, char *title, int item, int first)
{
	int l;
	int w;
	int num;
	int c;
	int cur = 1;
	int old = 1;
	int firstflag = 1;
	fd_set set;
	struct timeval timeout;
	
	mkmenubar(&num, menu);

	sizemenubar(&l, &w, title);

	/* failsafe positioning */

	if((y + l) >= LINES)
		y -= (y+l+1) - LINES;

	if((x + w) >= COLS)
		x -= (x+w+1) - COLS;

	MENU = newwin(l, w, y, x);	/* start (x, y) to (x+w, y+l) */

	dispmenu(boxflag, title, w, l);

	if( item != -1 && item > 0 && item < num)
		cur = old = item+1;
	
	noecho();
	crmode();
	keypad(MENU, TRUE);                   /* use special keys */
	
	for (;;)
	{
		delight(old);
		hilight(cur);

		/* timout handling for first keystroke */

		if(first && firstflag)
		{
			char tostring[MAXLINELEN];
			int secsleft = first;
			WINDOW *towin;

			sprintf(tostring, "Time left for first keypress: %3d seconds", secsleft);

			towin = newwin(3, strlen(tostring)+3, 9, 35);
			wborder(towin, 0, 0, 0, 0, 0, 0, 0, 0);
			mvwprintw(towin, 1, 1, "%s", tostring);
			wrefresh(towin);
			
			firstflag = 0;

			for(;;)
			{
				FD_ZERO(&set);
				FD_SET(STDIN_FILENO, &set);
				timeout.tv_sec = 1;
				timeout.tv_usec = 0;

				if((select(STDIN_FILENO + 1, &set, NULL, NULL, &timeout)) <= 0)
				{
					secsleft--;

					if(secsleft)
					{
						mvwprintw(towin, 1, 1, "Time left for first keypress: %3d seconds", secsleft);
						wrefresh(towin);
						continue;
					}
					else
					{
						wclear(towin);
						wrefresh(towin);
						delwin(towin);
				                wclear(MENU);
        		                        wrefresh(MENU);
						delwin(MENU);
						return(-1);
					}
				}
				else
				{
					break;
				}
			}
			wclear(towin);
			wrefresh(towin);
			delwin(towin);
		}

		c = wgetch(MENU);

		switch(c)
		{
			case KEY_UP:
				old = cur--;
				break;

			case KEY_DOWN:
				old = cur++;
				break;

			case CR:
			case LF:
		                wclear(MENU);
                                wrefresh(MENU);
				delwin(MENU);
				return(cur);
				break;

			default:
				if (c > '0' && c <= '9')
				{
					old = cur;
					cur = c - '0';
					if (cur > num)
						cur = num;
					if (cur < 1)
						cur = 1;
				}
				break;
		}
		if (cur > num)
			cur = 1;
		if (cur < 1)
			cur = num;
	}
}

/* EOF */
